/*
 * file: clp_cfg.h		*JZ*		28Nov98
 *
 *	CLP support - definitions for describing
 *		the installed solver(s)
 */
#ifndef CLP_CFG_H_INCLUDED
#define CLP_CFG_H_INCLUDED


#ifdef	__cplusplus
extern "C" {
#endif

#define	CLP_MAX_SOLVERS		4

#ifndef N_SOLVERS		/* N_SOLVERS also can be supplied externally! */
#if CONSTRAINT				/* (this is used in internal development) */
#define	N_SOLVERS	1
#else
#define	N_SOLVERS	0
#endif
#else
#if N_SOLVERS > CLP_MAX_SOLVERS
#error The runtime system cannot host more than 4 solvers
#endif
#endif


struct	ClpIfTable_s;
typedef	struct ClpIfTable_s	ClpIfTable;

typedef	enum {
	 ClpCoopMono = 0, ClpCoopXeno = 1, ClpCoopTolerant = 2, ClpCoopCooper = 3
} ClpCooperation;

typedef struct {

   ClpIfTable	const	*if_addr;	/* global table defined in the solver	*/
   char		const	*solver_name;	/* to be used in select_clp_solver/1	*/
   ClpCooperation	coop;		/* cooperativness - see the explanation	*/

} ClpSolverEntry;

typedef struct {

   size_t		n_entries;
#if N_SOLVERS
   ClpSolverEntry	solver[N_SOLVERS];
#endif

} ClpSolverDescr;

/*typedef const	ClpSolverEntry	ClpSolverDescr[CLP_MAX_SOLVERS];*/
 
extern	ClpSolverDescr	XxpSolverTable;


#ifdef	__cplusplus
}
#endif
/* --------------------------------------------- */
#endif /*CLP_CFG_H_INCLUDED*/
/* --------------------------------------------- */
/******************************
 NOTES:
   1. Prerequisite headers:
	'stdlib.h'
   2. The simple solution is to define the desctiptor table 
	as an array of CLP_MAX_SOLVERS solver entries, but
	it usually wastes static initialized space.
	The alternative, a structure with a counter and taylored
	array size, might cause trouble with C++ where type
	checking across modules is more strict (try it out!)
   3. Version info and description is removed from here; description
	is asked directly through the interface, it should/may include
	version number (or let them pass as separate output arguments?)
*******************************/
