/*
 *   File: csprolog.h
 *
 *   Header file for CS-Prolog II C interface
 *
 *   Copyright (C) 1997,  ML Ltd
 */
#ifndef CSPROLOG_H
#define CSPROLOG_H 1


#ifdef	__cplusplus
extern "C" {
#endif


#if __STDC__ || mips
#define ARGL1(x1) x1
#define ARGL2(x1,x2) x1,x2
#define ARGL3(x1,x2,x3) x1,x2,x3
#define ARGL4(x1,x2,x3,x4) x1,x2,x3,x4
#define ARGL5(x1,x2,x3,x4,x5) x1,x2,x3,x4,x5
#else
#define ARGL1(x1)
#define ARGL2(x1,x2)
#define ARGL3(x1,x2,x3)
#define ARGL4(x1,x2,x3,x4)
#define ARGL5(x1,x2,x3,x4,x5)
#endif

#if __STDC__
#define CONST const
#else
#define CONST
#endif

	 
typedef unsigned long	xxp_cell;
typedef xxp_cell	XxpCellArray[];
typedef double		CspFloatType;
typedef int		CspRetC;

typedef int (*UndoRoutine)(ARGL1(xxp_cell *));


typedef struct {
	    unsigned char  bc;
	    unsigned char  argno;
	    unsigned char  flags;
	    unsigned char  unused;
	    long           proc;
	       } extb_choice_point;


#define XXP_T_EMPTYREF	2
#if CONSTRAINT
#define XXP_T_RESTR	3
#endif
#define XXP_T_FLOAT	4
#define XXP_T_INT	5
#define XXP_T_SYMB	6
#define XXP_T_LIST	7
#define XXP_T_STRUCT	8


#define MAX_ATOM_LENGTH 4096

#define XXP_NIL         (xxp_cell)(15)



		/* Special codes - cannot be signalled, error kind is 0! */
#define XXP_E_FAIL              0
#define XXP_E_SUCCEED           1

		/* Real error codes: */
#define XXP_E_INSTANTIATION	11
#define XXP_E_TYPE		12
#define XXP_E_DOMAIN 		13
#define XXP_E_EXISTENCE 	14
#define XXP_E_PERMISSION	15
#define XXP_E_REPRESENTATION	16
#define XXP_E_SYNTAX_ERROR  	17
#define XXP_E_RESOURCE 		18
#define XXP_E_EVALUATION	19
#define XXP_E_CONSISTENCY	20
#define XXP_E_SYSTEM		21
#define XXP_E_INTERRUPT 	22
#define XXP_E_CLP_SYS		23

#define XXP_ERROR_KIND(ret)	(((ret) & 0x0FFF) >> 1)
#define XXP_ERROR_ARGNO(ret)	(((ret) >> 12) & 0xFF)


#define EXC_FLAG_CHARACTER              (1)
#define EXC_FLAG_CHARACTER_CODE		(2)
#define EXC_FLAG_IN_CHARACTER_CODE	(3)
#define EXC_FLAG_MAX_ARITY		(4)
#define EXC_FLAG_MAX_INTEGER		(5)
#define EXC_FLAG_MIN_INTEGER		(6)
#define EXC_FLAG_MAX_ATOM		(7)
#define EXC_FLAG_MAX_FORMAT		(8)
#define EXC_FLAG_MAX_MESSAGE		(9)
#define EXC_FLAG_MAX_TERM_SIZE		(10)
#define EXC_FLAG_MAX_CALL_IN_CLAUSE	(11)
#define EXC_FLAG_MAX_PATH_IN_CLAUSE	(12)
#define EXC_FLAG_MAX_CALL_IN_PATH	(13)
#define EXC_FLAG_MAX_CLAUSE		(14)

#define EXC_EVAL_FLOAT_OVERFLOW (1)
#define EXC_EVAL_INT_OVERFLOW	(2)
#define EXC_EVAL_UNDEFINED	(3)
#define EXC_EVAL_UNDERFLOW	(4)
#define EXC_EVAL_ZERO_DIVISOR	(5)
#define EXC_EVAL_BAD_FLOAT	(6)
#define EXC_EVAL_FORMAT		(7)

#define EXC_OTYPE_PROCEDURE 	(1)
#define EXC_OTYPE_SOURCE_SINK	(2)
#define EXC_OTYPE_STREAM	(3)
#define EXC_OTYPE_MODULE	(4)
#define EXC_OTYPE_EVENT		(5)
#define EXC_OTYPE_PROCESS	(6)
#define EXC_OTYPE_TERM		(7)
#define EXC_OTYPE_CHANNEL	(8)
#define EXC_OTYPE_CONTROL	(9)

#define EXC_OTYPE_NETOBJECT	(10)
#define EXC_OTYPE_NETHOST	(11)
#define EXC_OTYPE_NETSERVICE	(12)

#define EXC_OPER_ACCESS 	(1)
#define EXC_OPER_CREATE		(2)
#define EXC_OPER_INPUT		(3)
#define EXC_OPER_MODIFY		(4)
#define EXC_OPER_OPEN		(5)
#define EXC_OPER_OUTPUT		(6)
#define EXC_OPER_REPOSITION	(7)
#define EXC_OPER_REMOVE		(8)
#define EXC_OPER_PARALLEL	(9)
#define EXC_OPER_BIND		(10)
#define EXC_OPER_UNBIND		(11)


#define EXC_PERMTYPE_BINARY_STREAM	(1)
#define EXC_PERMTYPE_FLAG		(2)
#define EXC_PERMTYPE_OPERATOR		(3)
#define EXC_PERMTYPE_PAST_END_OF_STREAM	(4)
#define EXC_PERMTYPE_PRIVATE_PROCEDURE	(5)
#define EXC_PERMTYPE_STATIC_PROCEDURE	(6)
#define EXC_PERMTYPE_SOURCE_SINK	(7)
#define EXC_PERMTYPE_STREAM		(8)
#define EXC_PERMTYPE_TEXT_STREAM	(9)
#define EXC_PERMTYPE_PROCESS		(10)
#define EXC_PERMTYPE_CHANNEL		(11)
#define EXC_PERMTYPE_B_PROCEDURE	(12)
#define EXC_PERMTYPE_NB_PROCEDURE	(13)
#define EXC_PERMTYPE_LOCAL_PROC		(14)
#define EXC_PERMTYPE_NON_FLEX		(15)
#define EXC_PERMTYPE_PART_FLEX		(16)
#define EXC_PERMTYPE_WRONG_PROCEDURE	(17)
#define EXC_PERMTYPE_EVENT		(18)
#define EXC_PERMTYPE_VALUE		(19)
#define EXC_PERMTYPE_NETOBJ_NAME	(20)
#define EXC_PERMTYPE_NETOBJECT		(21)
#define EXC_PERMTYPE_NET_ATTRIBUTE	(22)
#define EXC_PERMTYPE_NET_BUFFERING_LIMIT (23)
#define EXC_PERMTYPE_BRACKET		(24)
#define EXC_PERMTYPE_CTRL_CONSTR	(25)
#define EXC_PERMTYPE_BLTIN_PROCEDURE	(26)
#define EXC_PERMTYPE_LOOP		(27)


#define EXC_RESOU_MEMORY 		(1)
#define EXC_RESOU_DISK_SPACE		(2)
#define EXC_RESOU_OPEN_STREAMS		(3)


#define EXC_VDOMA_CHARACTER_CODE_LIST	(1)
#define EXC_VDOMA_CLOSE_OPTION		(2)
#define EXC_VDOMA_FLAG_VALUE		(3)
#define EXC_VDOMA_IO_MODE		(4)
#define EXC_VDOMA_NON_EMPTY_LIST	(5)
#define EXC_VDOMA_NOT_LESS_THAN_ZERO	(6)
#define EXC_VDOMA_POSITIVE_NUMBER	(7)
#define EXC_VDOMA_OPERATOR_PRIORITY	(8)
#define EXC_VDOMA_OPERATOR_SPECIFIER	(9)
#define EXC_VDOMA_PROLOG_FLAG		(10)
#define EXC_VDOMA_BRACKET_PRIORITY	(11)
#define EXC_VDOMA_READ_OPTION		(12)
#define EXC_VDOMA_SOURCE_SINK		(13)
#define EXC_VDOMA_STREAM		(14)
#define EXC_VDOMA_STREAM_OPTION		(15)
#define EXC_VDOMA_STREAM_OR_ALIAS	(16)
#define EXC_VDOMA_STREAM_POSITION	(17)
#define EXC_VDOMA_STREAM_PROPERTY	(18)
#define EXC_VDOMA_WRITE_OPTION		(19)
#define EXC_VDOMA_UNIQUE		(20)
#define EXC_VDOMA_CHANNEL_MODE		(21)
#define EXC_VDOMA_RADIX			(22)
#define EXC_VDOMA_MAXDEPTH		(23)
#define EXC_VDOMA_TIMEOUT		(24)
#define EXC_VDOMA_QUEUE_SIZE		(25)
#define EXC_VDOMA_NET_CLOSE_MODE_OPTION	(26)
#define EXC_VDOMA_NET_INIT_MODE_OPTION	(27)
#define EXC_VDOMA_NET_PARTNER_LIMIT	(28)
#define EXC_VDOMA_OPTION		(29)
#define EXC_VDOMA_NETCONFIG_OPTION	(30)
#define EXC_VDOMA_EXPL_NETOBJ_NAME	(31)
#define EXC_VDOMA_IPADDR		(32)
#define EXC_VDOMA_TCP_PORT		(33)
#define EXC_VDOMA_NETCONFIG_ACTION	(34)
#define EXC_VDOMA_NET_ATTRIBUTE		(35)
#define EXC_VDOMA_NET_ATTR_VALUE	(36)
#define EXC_VDOMA_PREDICATE_MODE	(37)
#define EXC_VDOMA_MEDIATOR_KIND		(38)
#define EXC_VDOMA_HNMS_OPTION		(39)
#define EXC_VDOMA_BRACKET_NAME		(40)
#define EXC_VDOMA_MODULE_NAME		(41)
#define EXC_VDOMA_PROPER_LIST		(42)
#define EXC_VDOMA_GROUND_TERM		(43)

#define EXC_VDOMA_CLPEXT	EXC_VDOMA_GROUND_TERM
#define	EXC_VDOMA_CLP_RELATION		(EXC_VDOMA_CLPEXT + 1)
#define	EXC_VDOMA_CLP_LIN_EXPR		(EXC_VDOMA_CLPEXT + 2)
#define	EXC_VDOMA_CLP_LIN_EVAL		(EXC_VDOMA_CLPEXT + 3)
#define	EXC_VDOMA_CLP_FORMULA		(EXC_VDOMA_CLPEXT + 4)
#define	EXC_VDOMA_CLP_SOLVERS		(EXC_VDOMA_CLPEXT + 5)

#define EXC_VTYPE_VARIABLE 		XXP_T_EMPTYREF
#define EXC_VTYPE_CONSTR_VAR 		XXP_T_RESTR
#define EXC_VTYPE_ATOM			XXP_T_SYMB
#define EXC_VTYPE_INTEGER		XXP_T_INT
#define EXC_VTYPE_FLOAT 		XXP_T_FLOAT
#define EXC_VTYPE_COMPOUND		XXP_T_STRUCT
#define EXC_VTYPE_LIST		 	XXP_T_LIST
#define EXC_VTYPE_ATOMIC 		(101)
#define EXC_VTYPE_BYTE			(102)
#define EXC_VTYPE_CALLABLE		(103)
#define EXC_VTYPE_CHARACTER		(104)
#define EXC_VTYPE_EVALUABLE		(105)
#define EXC_VTYPE_IN_BYTE		(106)
#define EXC_VTYPE_IN_CHARACTER		(107)
#define EXC_VTYPE_NUMBER		(108)
#define EXC_VTYPE_PREDICATE_INDICATOR	(109)
#define EXC_VTYPE_TABLE_KEY		(110)
#define EXC_VTYPE_PERIODICITY		(111)
#define	EXC_VTYPE_CLP_EXPRESSION	(112)	/* struct, number, restr var, var */
#define	EXC_VTYPE_CLP_EVAL_EXPR		(113)	/* struct, number, restr var */
#define EXC_VTYPE_CLP_SOLVER_SEL	(114)	/* atom or integer	*/


#define EXC_CTYPE_NETADDR		(1)
#define EXC_CTYPE_SELFHOST		(2)
#define EXC_CTYPE_BRACKET_NAME		(3)


	/* Functions for working with exceptions: */

extern	CspRetC	 xxp_instantiation_error(ARGL2(unsigned,xxp_cell));
extern	CspRetC	 xxp_type_error(ARGL4(unsigned,unsigned,xxp_cell,xxp_cell));
extern	CspRetC	 xxp_domain_error(ARGL4(unsigned,unsigned,xxp_cell,xxp_cell));
extern	CspRetC	 xxp_existence_error(ARGL4(unsigned,unsigned,xxp_cell,xxp_cell));
extern	CspRetC	 xxp_permission_error(ARGL4(unsigned,unsigned,xxp_cell,xxp_cell));
extern	CspRetC	 xxp_representation_error(ARGL3(unsigned,unsigned,xxp_cell));
extern	CspRetC	 xxp_syntax_error(ARGL2(xxp_cell,xxp_cell));
extern	CspRetC	 xxp_resource_error(ARGL2(unsigned,xxp_cell));
extern	CspRetC	 xxp_evaluation_error(ARGL3(unsigned,unsigned,xxp_cell));
extern	CspRetC	 xxp_system_error(ARGL1(xxp_cell));
extern	CspRetC	 xxp_consistency_error(ARGL3(unsigned,xxp_cell,xxp_cell));
#if CONSTRAINT
extern	CspRetC	 xxp_clp_sys_error(ARGL1(xxp_cell));
#endif
extern	CspRetC	 xxp_interrupt(ARGL2(unsigned,xxp_cell));

extern	void	 xxp_reset_error_flags(ARGL1(void));
extern	xxp_cell xxp_construct_builtin_call(ARGL1(void));
extern	void	 xxp_signal_prepared(ARGL2(xxp_cell,xxp_cell));
extern	void	 xxp_signal(ARGL1(CspRetC));
extern	xxp_cell xxp_get_error_term(ARGL1(CspRetC));
extern	xxp_cell xxp_get_other_for_error(ARGL1(CspRetC));


	/* Functions for creating and accessing Prolog terms: */

extern	xxp_cell xxp_arg(ARGL1(unsigned));
extern	int	 xxp_type(ARGL1(xxp_cell));
extern	unsigned xxp_raw_type(ARGL1(xxp_cell));
extern	xxp_cell xxp_list_head(ARGL1(xxp_cell));
extern	xxp_cell xxp_list_tail(ARGL1(xxp_cell));
extern	xxp_cell xxp_struct_functor(ARGL1(xxp_cell));
extern	unsigned xxp_struct_arity(ARGL1(xxp_cell));
extern	xxp_cell xxp_struct_arg(ARGL2(unsigned,xxp_cell));
extern	char	 *xxp_symb_chars(ARGL1(xxp_cell));
extern	CspFloatType xxp_float_val(ARGL1(xxp_cell));
extern	CspFloatType xxp_checked_float_val(ARGL3(xxp_cell, unsigned, CspRetC *));
extern	xxp_cell xxp_mk_emptyref(ARGL1(void));
extern	xxp_cell xxp_mk_list(ARGL2(xxp_cell,xxp_cell));
extern	xxp_cell xxp_mk_struct(ARGL3(unsigned,xxp_cell,XxpCellArray));
extern	xxp_cell xxp_mk_symb(ARGL1(CONST char *));
extern	xxp_cell xxp_mk_functor(ARGL2(CONST char *,unsigned));
extern	xxp_cell xxp_mk_float(ARGL1(CspFloatType));
extern	xxp_cell xxp_checked_mk_int(ARGL3(long, unsigned, CspRetC *));
			/* Term unification: */
extern	int	 xxp_unify(ARGL2(xxp_cell,xxp_cell));
extern	int	 xxp_unify_with_occurs_check(ARGL2(xxp_cell,xxp_cell));
extern	int	 xxp_safe_unify(ARGL2(xxp_cell,xxp_cell));


	/* Functions for non-deterministic predicates: */

extern	int	 xxp_first_call(ARGL1(void));
extern	void 	 xxp_create_choice_point(ARGL2(unsigned,extb_choice_point *));
extern	void 	 xxp_set_choice_point_arg(ARGL2(unsigned,xxp_cell));
extern	void	 xxp_destroy_choice_point(ARGL1(void));


	/* Functions for bacltrackable predicates (trail manipulation): */

extern	int 	 xxp_register_user_trail_block(ARGL3(int,long,UndoRoutine));
extern	void 	 xxp_make_trail_note(ARGL2(int,xxp_cell *));


	/* Functions for memory management: */

extern	char	 *xxp_allocate_user_block(ARGL1(unsigned));
extern	void	 xxp_free_user_block(ARGL1(char *));


	/* Functions for generating CSP events and interrupts: */

extern	int	 xxp_generate_event(ARGL2(xxp_cell,xxp_cell));
extern	int	 xxp_cause_interrupt(ARGL2(xxp_cell,xxp_cell));

extern	int	xxp_active_process_id(int *processor_idp);


	/* Calling a Prolog predicate from C: */

extern	int	 xxp_call_prolog_from_C(ARGL1(xxp_cell *));


	/* Functions transformed into macros: */

#define xxp_mk_int(i) ((xxp_cell)((((unsigned long)(i)<<5)>>1)|0x03))


#if _transputer
extern	long	TP_sign_extend(ARGL2(long,long));
#define xxp_int_val(cell) TP_sign_extend((long)((cell)<<1)>>5,0x04000000)
#else
#define xxp_int_val(cell) ((long)((cell)<<1)>>5)
#endif


#ifdef	__cplusplus
}
#endif


#endif /* #ifndef CSPROLOG_H */
/* --------------------------------------------- */
/* --------------------------------------------- */
