/*
 * file: pzclp.pro
 *	- The 'maximal' integral solution
 *		within the designated region is to be obtained
 */

#define ePS		5.0E-8
#define aLL_SOLUTIONS	1

%#define ShowMap(VL, Tag)	show_map((VL), (Tag))
%#define TracE(Fmt, Lst)		format((Fmt), (Lst))
#define ShowMap(VL, Tag)	true
#define TracE(Fmt, Lst)		true

:-module(inequality,[main_goal/0]).

main_goal:-
      select_clp_solver,
%	clp_debug_mode(255),				% Full
	clp_debug_mode(128),				% Trace messages only
	clp_constraint([ X + Y >= 2.1,			% Define allowed region
                        -1.20 * X + Y >= 0.0,
                        -0.35 * X + Y =< 2.5]),
	   ShowMap([[X, 'X'], [Y, 'Y']], 'Initial'),
           solution(X,Y,XV,YV),
%          int_solution(XV), int_solution(YV),
#if aLL_SOLUTIONS
           format('Integer solution (ordered): (X=~3f,Y=~3f)~n', [XV, YV]),
	   fail
#else
           format('Maximal integer solution (ordered): (X=~3f,Y=~3f)~n', [XV, YV])
#endif
	   .

main_goal:-
	format('The problem has no (more) integral solution(s)~n', []).

solution(X, Y, XV, YV):-
	clp_debug_mode(192),			% Trace messages + status print
        clp_max(X, Xupb),			% Find out the maximal X possible
	clp_debug_mode(128),			% Trace messages only
          clp_min(X, Xlwb),			% Find out the minimal X possible
	  TryX is floor(Xupb),			%  - first candidate level
	  LastX is ceiling(Xlwb),
	  solve(X, Y, TryX, LastX, XV, YV).

solve(X, Y, TryX, LastX, _, _) :-
	ShowMap([[X, 'X'], [Y, 'Y']], 'Solve begin'),
	TryX < LastX, !, fail.
solve(X, Y, TryX, _LastX, TryX, YV) :-
%	TracE('fixing X =< ~w~n', [TryX + ePS]),
%	clp_constraint([X =< TryX + ePS]),		% - try to fix it at that value
	TracE('fixing X = ~w~n', [TryX]),
%	clp_constraint([X = TryX]),			% - try to fix it at that value
	X = TryX,	%	- try to fix it at that value
	  ShowMap([[X, 'X'], [Y, 'Y']], 'X limited'),
          clp_max(Y, Ymax),				% - what is the corresponding maximal Y?
	  TryY is floor(Ymax),
		% We could check TryY against the minimum, but it is not cheaper!
%	  TracE('fixing Y =< ~w~n', [TryY + ePS]),
%	  clp_constraint([Y =< TryY + ePS]),		% Fix at integral value
	  TracE('fixing Y = ~w~n', [TryY]),
%	  clp_constraint([Y = TryY]),			% Use constraint
	  Y = TryY,					% Use unification
	  ShowMap([[X, 'X'], [Y, 'Y']], 'Y 1st limit'),
	  clp_max(Y, YV),	      %  - *** Error: instead of 3, max returns something below 0 (cut X?) ***
	  (YV >= TryY - ePS -> true;
		format('*Error* max Y is too low: ~w instead of ~w~n', [YV, TryY + ePS])
	  ),
	  int_solution(YV),
	  clp_min(Y, Ylow),
	  TracE('min Y = ~w~n', [Ylow]),
	  Ylow =< YV,
	  clp_constraint([X =< TryX + ePS, Y =< YV + ePS,
		X >= TryX - ePS, Y >= YV - ePS])	% (Final check)
#if \+ aLL_SOLUTIONS
	  ,!
#endif
	  .
solve(X, Y, TryX, LastX, XV, YV) :-
	NextX is TryX - 1,
	  solve(X, Y, NextX, LastX, XV, YV).

int_solution(Z) :-
        (A is Z - floor(Z) ; A is ceiling(Z) - Z),
	   A < 2 * ePS, !.


%   For testing:

show_map(L, Spot) :-
	format('~n   Map at \'~w\':~n', [Spot]),
	show_varlist(L).

show_varlist([]) :- !, nl.
show_varlist([[V,Tag]|T]) :-
	format('Variable ''~w''~n', [Tag]),
	  clp_type(V, Type),
	  (Type \= free, Type \= upbnd -> clp_min(V, Min); true),
	  (Type \= free, Type \= lobnd -> clp_max(V, Max);  true),
	  %clp_value(V, Val),		% *** Requires list!!! ***
	  clp_value([V], [Val]),
	  format(' Type: ~w, Min: ~w, Max: ~w, Value: ~w~n', [Type, Min, Max, Val]),
	  show_varlist(T).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NOTES:
%  1. clp_value causes type error ('number'). - Cause: it expects list (coordinated?) !!!
%  2. When fixing value as constant, strange things happen.
%  3. After limiting or fixing Y the model becomes invalid!
%  4. On backtracking the values are not restored correctly.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





