:-module(convert_dcg, [main_goal/0]).

:-import(dcg_converter:expand_term/2).

% :-op(1200, xfx, -->).

#define sPEC_VAR_ATOM		'__S'
#define sPEC_VAR_ATOM_CHARS	'_','_','S'

main_goal :-
	op(1200, xfx, -->),
	cpu_time(T1),
		(catch(go, Err, report(Err))
		; cpu_time(T2), T is (T2 - T1) / 100, 
			format(user_error, '~NExecution time: ~2f sec.~n', [T])
		).

#if XYZ_NEW_BACKTRACK || 1	/* (changed for backtracking cycle) */
go :-
      repeat,
	read_term(X, [variable_names(V)]), handle_op(X),
	(X = end_of_file -> !, write('''C''([X|S], X, S).\n')
	 ; expand_term(X, Y) -> tidy_variables(Y, V, VV),	
		write_expanded_term(Y, VV), fail
	 ; throw(expansion_failed_for(X))
	),
	fail.
#else
go :-
	read_term(X, [variable_names(V)]), handle_op(X),
	(X = end_of_file -> write('''C''([X|S], X, S).\n'), fail
	 ; expand_term(X, Y) -> tidy_variables(Y, V, VV),	
		write_expanded_term(Y, VV)
	 ; throw(expansion_failed_for(X))),
	go.
#endif

handle_op(X) :-
	X = (:- op(O1, O2, O3)) -> op(O1, O2, O3);
	true.

write_expanded_term((H :- B), VV) :-
	!, wr_term(H, VV), write(' :-'), nl,
		write_expanded_body(B, VV).
write_expanded_term(H, VV) :-
	wr_term(H, VV), write('.'), nl.

write_expanded_body((C ,B), VV) :-
	!, write('\t'), wr_term(C, VV), write(','), nl,
		write_expanded_body(B, VV).
write_expanded_body(B, VV) :-
	write('\t'), wr_term(B, VV), write('.'), nl.

wr_term(T, VV) :-
	write_term(T, [variable_names(VV), quoted]).

tidy_variables(Y, V, VV) :-
	insert_S(Y, V, VVV),
	  build_var_table(Y, VVV, VV, 0, _).

build_var_table(Y, V, VV, N, NN) :-
	compound(Y) -> Y =.. [_ | L], build_var_table_for_list(L, V, VV, N, NN);
	var(Y), not(var_member(Y, V)) -> number_chars(N, CHL), atom_chars(A, [sPEC_VAR_ATOM_CHARS | CHL]),
						VV = [Y = A | V], NN is N + 1;
	VV = V, NN = N.

build_var_table_for_list([], V, V, N, N).
build_var_table_for_list([Y | L], V, VV, N, NN) :-
	build_var_table(Y, V, VVV, N, NNN), build_var_table_for_list(L, VVV, VV, NNN, NN).

var_member(Y, [V = _ | L]) :-
	Y == V -> true;
	var_member(Y, L).

insert_S((H :- _), V, VV) :-
	compound(H), H =.. L, last_elem(L, Var) , !,
		VV = [Var = sPEC_VAR_ATOM | V].
insert_S(_, V, V).

last_elem([X], X):- !.
last_elem([_ | L], X) :-
	last_elem(L, X).

report(Err) :-
	format(user_error, '~n''~q'' error occurred during execution~n',[Err]).
