:-module(dcg_converter, [expand_term/2, 'C'/3]).

:-op(1200, xfx, -->).

%
% Copied from SWI Prolog
%

expand_term(Term, Expanded):-
	'$translate_rule'(Term, Expanded), !.
expand_term(Term, Term).
	

		/********************************
		*        GRAMMAR RULES          *
		*********************************/

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
The DCG compiler. The original code was copied from C-Prolog and written
by Fernando Pereira, EDCAAD, Edinburgh,  1984.   Since  then many people
have modified and extended this code. It's a nice mess now and it should
be redone from scratch. I won't be doing   this  before I get a complete
spec explaining all an implementor needs to   know  about DCG. I'm a too
basic user of this facility myself (though   I  learned some tricks from
people reporting bugs :-)

The original version contained  '$t_tidy'/2  to   convert  ((a,b),  c)  to
(a,(b,c)), but as the  SWI-Prolog  compiler   doesn't  really  care (the
resulting code is simply the same), I've removed that.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

'$translate_rule'((LP-->List), H) :-
	proper_list(List), !,
	(   List = []
	->  '$t_head'(LP, S, S, H)
	;   List = [X]
	->  '$t_head'(LP, [X|S], S, H)
	;   append(List, SR, S),
	    '$extend'([S, SR], LP, H)
	), !.
'$translate_rule'((LP-->RP), (H:-B)):-
	'$t_head'(LP, S, SR, H),
	'$t_body'(RP, S, SR, B).

'$tailvar'(X, X) :-
	var(X), !.
'$tailvar'([_|T], V) :-
	'$tailvar'(T, V).

'$t_head'((LP, List), S, SR, H) :-
	append(List, SR, List2), !,
	'$extend'([S, List2], LP, H).
'$t_head'(LP, S, SR, H) :-
	'$extend'([S, SR], LP, H).


'$t_body'(Var, S, SR, phrase(Var, S, SR)) :-
	var(Var), !.
'$t_body'(List, S, SR, C) :-
	proper_list(List), !,
	(   List = []
	->  C = (S=SR)
	;   List = [X]
	->  C = 'C'(S, X, SR)
	;   C = append(List, SR, S)
	).
'$t_body'(List, S, SR, C) :-
	List = [_|_], !,
	C = append(List, SR, S).
'$t_body'(!, S, S, !) :- !.
'$t_body'({T}, S, SR, (T, SR = S)) :- !.
'$t_body'((T, R), S, SR, (Tt, Rt)) :- !,
	'$t_body'(T, S, SR1, Tt),
	'$t_body'(R, SR1, SR, Rt).
'$t_body'((T;R), S, SR, (Tt;Rt)) :- !,
	'$t_body'(T, S, S1, T1), '$t_fill'(S, SR, S1, T1, Tt),
	'$t_body'(R, S, S2, R1), '$t_fill'(S, SR, S2, R1, Rt).
#if XYZ_OLD_KEEP		/* '|' is not an operator in CSP */
'$t_body'((T|R), S, SR, (Tt;Rt)) :- !,
	'$t_body'(T, S, S1, T1), '$t_fill'(S, SR, S1, T1, Tt),
	'$t_body'(R, S, S2, R1), '$t_fill'(S, SR, S2, R1, Rt).
#endif
'$t_body'((C->T;E), S, SR, (Ct->Tt;Et)) :- !,
	'$t_body'(C, S, S1, Ct),
	'$t_body'(T, S1, S2, T1), '$t_fill'(S, SR, S2, T1, Tt),
	'$t_body'(E, S1, S3, E1), '$t_fill'(S, SR, S3, E1, Et).
#if XYZ_OLD_KEEP		/* '|' is not an operator in CSP */
'$t_body'((C->T|E), S, SR, (Ct->Tt;Et)) :- !,
	'$t_body'(C, S, S1, Ct),
	'$t_body'(T, S1, S2, T1), '$t_fill'(S, SR, S2, T1, Tt),
	'$t_body'(E, S1, S3, E1), '$t_fill'(S, SR, S3, E1, Et).
#endif
'$t_body'((C->T), S, SR, (Ct->Tt)) :- !,
	'$t_body'(C, S, SR1, Ct),
	'$t_body'(T, SR1, SR, Tt).
'$t_body'((\+ C), S, SR, (\+ Ct)) :- !,
	'$t_body'(C, S, SR, Ct).
'$t_body'(T, S, SR, Tt) :-
	'$extend'([S, SR], T, Tt).


'$t_fill'(S, SR, S1, T, (T, SR=S)) :-
	S1 == S, !.
'$t_fill'(_S, SR, SR, T, T).


'$extend'(More, OldT, NewT) :-
	OldT =.. OldL,
	append(OldL, More, NewL),
	NewT =.. NewL.

'C'([X|S], X, S).

append([], L, L).
append([X | L1], L2, [X | L3]) :-
	append(L1, L2, L3).

proper_list([]).
proper_list([X | L]):-
	nonvar(X), proper_list(L).
