:- module(pdss_mod,[main_goal/0]).

:-import([csp_debug:xxp_step_trace/1]).

main_goal:- 
     protected(pdss,pdss_handler,0).

pdss_handler(ERROR,INFO,_,1,pdss_mod $: pdss):-
  format(user_output,'~N**** ~w **** ~w *****~nXPDS restarted...~2n',[ERROR, INFO]).

pdss:- 
  command;
  pdss.

command:- 
  flush_input(user_input),
    format(user_output,'~Nxpds: ',[]), tread_token(user_input,COM), 
    legal(COM), xpds_exec(COM), !, end(COM).

end(bye).

legal(COM):- 
  is_member(COM,[e,c,r,x,run,tr,body,list,bye]), !;
  write(user_output,'Illegal command'), nl, fail.

is_member(X,[X|_]).
is_member(X,[_|L]):- 
  is_member(X,L).

xpds_exec(bye).
xpds_exec(e):-
  read(user_input,CL), assertz(CL).
xpds_exec(body):-
  read_body.
xpds_exec(c):-
  tread_token(user_input,FN), 
     open(FN,read,_X,[alias(fil)]), readfile, close(fil),
     write(user_output,consulted),nl.
xpds_exec(r):-
  xpds_exec_or_trace(run).
xpds_exec(x):-
  xpds_exec_or_trace(exec).
xpds_exec(tr):-
  xpds_exec_or_trace(trace).
xpds_exec(run):-
  run_mode.
xpds_exec(list):-
  list_pred.
run_mode:-
  format(user_output,'~Nrun: ',[]), flush_input(user_input),
    read_term(user_input,CL,[variable_names(VN)]), 
      (CL = end -> true;
       xpds_execute(run,CL,VN) -> fail).
run_mode:-
       run_mode.

xpds_exec_or_trace(IS_TR):-
  read_term(user_input,CL,[variable_names(VN)]), xpds_execute(IS_TR,CL,VN), !.

xpds_execute(IS_TR,CL,VN):-
   call_or_trace(IS_TR,CL), 
     (IS_TR = run ->  wr_succeed(VN);
      IS_TR = exec ->  wr_exec_succeed(VN);
      true).
xpds_execute(_,_,_):-
  format(user_output,'~Nfailed~n',[]).

call_or_trace(run,CL):-
  !, CL.
call_or_trace(exec,CL):-
  !, CL.
call_or_trace(trace,CL):-
  xxp_step_trace(pdss_mod $: CL).
  
wr_succeed([]):- 
  !, format(user_output,'~Nsucceeded~n',[]).
wr_succeed(VN):-
  write_vn(VN), 
    write(user_output,' ? '), get_continuation.
    
get_continuation:-    
  flush_input(user_input),
    get_code(user_input,Y), flush_input(user_input),
      (Y =< 0' , !, write(user_output,yes);
       Y = 0';, !, fail;
       write(user_output,'";" for more choices, otherwise <return>:' ),
         get_continuation).
    
wr_exec_succeed([]):- 
  !, format(user_output,'~Nsucceeded~n',[]).
wr_exec_succeed(VN):-
  write_vn(VN).

write_vn([]).
write_vn([X=Y|L]):-
  format(user_output,'~N\t~a~t~15|= ~q',[Y,X]),
    write_vn(L).

%portray(X) :- X is real, format('~18f',[X]).
    
readfile:-
    read_clause(X), add_clause_or_op(X), disp_add(X),
      readfile;
    true.

add_clause_or_op((main_goal :- _X)):-
   !, format(user_output,'~Nmain_goal/0 definition ignored~n',[]).
add_clause_or_op((main_goal(_Y) :- _X)):-
   !, format(user_output,'~Nmain_goal/1 definition ignored~n',[]).
add_clause_or_op((:- op(X,Y,Z))):-
  !, op(X,Y,Z).
add_clause_or_op((:- X)):-
   !, format(user_output,'~NDeclaration ~w ignored~n',[X]).
add_clause_or_op(X):-
  assertz(X).  

disp_add((main_goal:- _Y)):-
  !.
disp_add((main_goal(_X):- _Y)):-
  !.
disp_add((:- op(X,Y,Z))):-
  write_name(op(X,Y,Z)).
disp_add((:- _Y)):-
  !.
disp_add((X:-_Y)):-
  !, write_name(X).
disp_add(X):-
  write_name(X).
  
write_name(X):-
  functor(X,FU,AR),
    format(user_output,'~a/~d~n',[FU,AR]).

read_clause(X):-
  read(fil,X), X \= end_of_file.
  
read_body:-
  write(user_output,'body: '), read(user_input,X),  not_end_body(X),
     assertz(X), read_body.
read_body.

not_end_body(end):- 
  !, fail.
not_end_body(_X).

list_pred:-
  read(user_input,FU),
   (FU = N/A ->  functor(HEAD,N,A),
     (clause(HEAD,BODY), format(user_output,'~w~n~8|~w~n',[HEAD,BODY]), fail;
      true);
    write(user_output,'Functor needed'),nl).

